@@ -26,14 +26,17 @@ see duplicity's README for details
 """
 
 import os
+import sys
 import types
 import tempfile
 import re
 import gzip
 import locale
+import platform
 
 from duplicity import globals
 from duplicity import gpginterface
+from duplicity import log
 from duplicity import tempdir
 from duplicity import util
 
@@ -66,8 +69,7 @@ class GPGProfile:
         passphrase is the passphrase.  If it is None (not ""), assume
         it hasn't been set.  sign_key can be blank if no signing is
         indicated, and recipients should be a list of keys.  For all
-        keys, the format should be an 8 character hex key like
-        'AA0E73D2'.
+        keys, the format should be an hex key like 'AA0E73D2'.
         """
         assert passphrase is None or isinstance(passphrase, types.StringType)
 
@@ -87,6 +89,28 @@ class GPGProfile:
         else:
             self.hidden_recipients = []
 
+        self.gpg_version = self.get_gpg_version(globals.gpg_binary)
+
+    _version_re = re.compile(r'^gpg.*\(GnuPG(?:/MacGPG2)?\) (?P<maj>[0-9]+)\.(?P<min>[0-9]+)\.(?P<bug>[0-9]+)(-.+)?$')
+
+    def get_gpg_version(self, binary):
+        gnupg = gpginterface.GnuPG()
+        if binary is not None:
+            gnupg.call = binary
+
+        # user supplied options
+        if globals.gpg_options:
+            for opt in globals.gpg_options.split():
+                gnupg.options.extra_args.append(opt)
+
+        # get gpg version
+        res = gnupg.run(["--version"], create_fhs=["stdout"])
+        line = res.handles["stdout"].readline().rstrip()
+        m = self._version_re.search(line)
+        if m is not None:
+            return (int(m.group("maj")), int(m.group("min")), int(m.group("bug")))
+        raise GPGError("failed to determine gnupg version of %s from %s" % (binary, line))
+
 
 class GPGFile:
     """
@@ -114,10 +138,32 @@ class GPGFile:
 
         # Start GPG process - copied from GnuPGInterface docstring.
         gnupg = gpginterface.GnuPG()
+        # overrides default gpg binary 'gpg'
+        if globals.gpg_binary is not None:
+            gnupg.call = globals.gpg_binary
         gnupg.options.meta_interactive = 0
         gnupg.options.extra_args.append('--no-secmem-warning')
-        if globals.use_agent:
-            gnupg.options.extra_args.append('--use-agent')
+        gnupg.options.extra_args.append('--ignore-mdc-error')
+
+        # Support three versions of gpg present 1.x, 2.0.x, 2.1.x
+        if profile.gpg_version[:1] == (1,):
+            if globals.use_agent:
+                # gpg1 agent use is optional
+                gnupg.options.extra_args.append('--use-agent')
+
+        elif profile.gpg_version[:2] == (2, 0):
+            pass
+
+        elif profile.gpg_version[:2] >= (2, 1):
+            if not globals.use_agent:
+                # This forces gpg2 to ignore the agent.
+                # Necessary to enforce truly non-interactive operation.
+                gnupg.options.extra_args.append('--pinentry-mode=loopback')
+
+        else:
+            raise GPGError("Unsupported GNUPG version, %s" % profile.gpg_version)
+
+        # user supplied options
         if globals.gpg_options:
             for opt in globals.gpg_options.split():
                 gnupg.options.extra_args.append(opt)
@@ -158,7 +204,7 @@ class GPGFile:
                            attach_fhs={'stdout': encrypt_path.open("wb"),
                                        'stderr': self.stderr_fp,
                                        'logger': self.logger_fp})
-            if not(globals.use_agent):
+            if not globals.use_agent:
                 p1.handles['passphrase'].write(passphrase)
                 p1.handles['passphrase'].close()
             self.gpg_input = p1.handles['stdin']
@@ -217,7 +263,10 @@ class GPGFile:
         for fp in (self.logger_fp, self.stderr_fp):
             fp.seek(0)
             for line in fp:
-                msg += unicode(line.strip(), locale.getpreferredencoding(), 'replace') + u"\n"
+                try:
+                    msg += unicode(line.strip(), locale.getpreferredencoding(), 'replace') + u"\n"
+                except Exception as e:
+                    msg += line.strip() + u"\n"
         msg += u"===== End GnuPG log =====\n"
         if not (msg.find(u"invalid packet (ctb=14)") > -1):
             raise GPGError(msg)
@@ -260,7 +309,7 @@ class GPGFile:
 
     def set_signature(self):
         """
-        Set self.signature to 8 character signature keyID
+        Set self.signature to signature keyID
 
         This only applies to decrypted files.  If the file was not
         signed, set self.signature to None.
@@ -273,11 +322,11 @@ class GPGFile:
             self.signature = None
         else:
             assert len(match.group(1)) >= 8
-            self.signature = match.group(1)[-8:]
+            self.signature = match.group(1)
 
     def get_signature(self):
         """
-        Return 8 character keyID of signature, or None if none
+        Return  keyID of signature, or None if none
         """
         assert self.closed
         return self.signature
@@ -354,9 +403,7 @@ def GPGWriteFile(block_iter, filename, profile,
         raise
 
 
-def GzipWriteFile(block_iter, filename,
-                  size=200 * 1024 * 1024,
-                  max_footer_size=16 * 1024):
+def GzipWriteFile(block_iter, filename, size=200 * 1024 * 1024, gzipped=True):
     """
     Write gzipped compressed file of given size
 
@@ -385,7 +432,12 @@ def GzipWriteFile(block_iter, filename,
             return self.fileobj.close()
 
     file_counted = FileCounted(open(filename, "wb"))
-    gzip_file = gzip.GzipFile(None, "wb", 6, file_counted)
+
+    # if gzipped wrap with GzipFile else plain file out
+    if gzipped:
+        outfile = gzip.GzipFile(None, "wb", 6, file_counted)
+    else:
+        outfile = file_counted
     at_end_of_blockiter = 0
     while True:
         bytes_to_go = size - file_counted.byte_count
@@ -396,12 +448,27 @@ def GzipWriteFile(block_iter, filename,
         except StopIteration:
             at_end_of_blockiter = 1
             break
-        gzip_file.write(new_block.data)
+        outfile.write(new_block.data)
 
-    assert not gzip_file.close() and not file_counted.close()
+    assert not outfile.close() and not file_counted.close()
     return at_end_of_blockiter
 
 
+def PlainWriteFile(block_iter, filename, size=200 * 1024 * 1024, gzipped=False):
+    """
+    Write plain uncompressed file of given size
+
+    This is like the earlier GPGWriteFile except it writes a gzipped
+    file instead of a gpg'd file.  This function is somewhat out of
+    place, because it doesn't deal with GPG at all, but it is very
+    similar to GPGWriteFile so they might as well be defined together.
+
+    The input requirements on block_iter and the output is the same as
+    GPGWriteFile (returns true if wrote until end of block_iter).
+    """
+    return GzipWriteFile(block_iter, filename, size, gzipped)
+
+
 def get_hash(hash, path, hex=1):
     """
     Return hash of path
